#include "adif.h"
#include "dirdialog.h"
#include "configparams.h"
#include <QFileInfo>


sQSO qsoFields[NUMBEROFQSOFIELDS]=
{
  {"ADDRESS",        NULL},
  {"AGE",        NULL},
  {"A_INDEX",        NULL},
  {"ANT_AZ",        NULL},
  {"ANT_EL",        NULL},
  {"ANT_PATH",        NULL},
  {"ARRL_SECT",        NULL},
  {"BAND",        NULL},
  {"BAND_RX",        NULL},
  {"CALL",        NULL},
  {"CHECK",        NULL},
  {"CLASS",        NULL},
  {"CNTY",        NULL},
  {"COMMENT",        NULL},
  {"CONT",        NULL},
  {"CONTACTED_OP",        NULL},
  {"CONTEST_ID",        NULL},
  {"CREDIT_SUBMITTED",        NULL},
  {"CREDIT_GRANTED",        NULL},
  {"COUNTRY",        NULL},
  {"CQZ",        NULL},
  {"DISTANCE",        NULL},
  {"DXCC",        NULL},
  {"EMAIL",        NULL},
  {"EQ_CALL",        NULL},
  {"EQSL_QSLRDATE",        NULL},
  {"EQSL_QSLSDATE",        NULL},
  {"EQSL_QSL_RCVD",        NULL},
  {"EQSL_QSL_SENT",        NULL},
  {"FORCE_INIT",        NULL},
  {"FREQ",        NULL},
  {"FREQ_RX",        NULL},
  {"GRIDSQUARE",        NULL},
  {"IOTA",        NULL},
  {"IOTA_ISLAND_ID",        NULL},
  {"ITUZ",        NULL},
  {"K_INDEX",        NULL},
  {"LAT",        NULL},
  {"LON",        NULL},
  {"LOTW_QSLRDATE",        NULL},
  {"LOTW_QSLSDATE",        NULL},
  {"LOTW_QSL_RCVD",        NULL},
  {"LOTW_QSL_SENT",        NULL},
  {"MAX_BURSTS",        NULL},
  {"MODE",        NULL},
  {"MS_SHOWER",        NULL},
  {"MY_CITY",        NULL},
  {"MY_CNTY",        NULL},
  {"MY_COUNTRY",        NULL},
  {"MY_CQ_ZONE",        NULL},
  {"MY_GRIDSQUARE",        NULL},
  {"MY_IOTA",        NULL},
  {"MY_IOTA_ISLAND_ID",        NULL},
  {"MY_ITU_ZONE",        NULL},
  {"MY_LAT",        NULL},
  {"MY_LON",        NULL},
  {"MY_NAME",        NULL},
  {"MY_POSTAL_CODE",        NULL},
  {"MY_RIG",        NULL},
  {"MY_SIG",        NULL},
  {"MY_SIG_INFO",        NULL},
  {"MY_STATE",        NULL},
  {"MY_STREET",        NULL},
  {"NAME",        NULL},
  {"NOTES",        NULL},
  {"NR_BURSTS",        NULL},
  {"NR_PINGS",        NULL},
  {"OPERATOR",        NULL},
  {"OWNER_CALLSIGN",        NULL},
  {"PFX",        NULL},
  {"PRECEDENCE",        NULL},
  {"PROP_MODE",        NULL},
  {"PUBLIC_KEY",        NULL},
  {"QSLMSG",        NULL},
  {"QSLRDATE",        NULL},
  {"QSLSDATE",        NULL},
  {"QSL_RCVD",        NULL},
  {"QSL_RCVD_VIA",        NULL},
  {"QSL_SENT",        NULL},
  {"QSL_SENT_VIA",        NULL},
  {"QSL_VIA",        NULL},
  {"QSO_COMPLETE",        NULL},
  {"QSO_DATE",        NULL},
  {"QSO_DATE_OFF",        NULL},
  {"QSO_RANDOM",        NULL},
  {"QTH",        NULL},
  {"RIG",        NULL},
  {"RST_RCVD",        NULL},
  {"RST_SENT",        NULL},
  {"RX_PWR",        NULL},
  {"SAT_MODE",        NULL},
  {"SAT_NAME",        NULL},
  {"SFI",        NULL},
  {"SIG",        NULL},
  {"SIG_INFO",        NULL},
  {"SRX",        NULL},
  {"SRX_STRING",        NULL},
  {"STATE",        NULL},
  {"STATION_CALLSIGN",        NULL},
  {"STX",        NULL},
  {"STX_STRING",        NULL},
  {"SWL",        NULL},
  {"TEN_TEN",        NULL},
  {"TIME_OFF",        NULL},
  {"TIME_ON",        NULL},
  {"TX_PWR",        NULL},
  {"WEB",       NULL}
};

adif::adif()
{
  int i;
  ts=NULL;
  for(i=0;i<NUMBEROFQSOFIELDS;i++)
    {
      qsoFields[i].value=new QString;
    }
}

adif::~adif()
{
  if(ts) delete ts;
  iofile.close();
}

QStringList adif::getFieldList(bool tagged)
{
  int i;
  QStringList sl;
  for(i=0;i<NUMBEROFQSOFIELDS;i++)
    {
      if(tagged) sl.append(QString("<%1>").arg(qsoFields[i].tag));
      else sl.append(QString("%1").arg(qsoFields[i].tag));
    }
  return sl;
}

eerrorType adif::openFile()
{
  dirDialog dd(mainWindowPtr,"Import Adif");
  iofn=dd.openFileName(importPath,"ADIF(*.adi *.ADI *.adif *.ADIF);;ALL(*.*)");
  if (( iofn.isEmpty() ) || (iofn==NULL) ) return CANCELED;
  iofile.setFileName(iofn);
  QFileInfo fn(iofn);
  if(!iofile.open( QIODevice::ReadOnly ))
    {
      return CRITICAL;
    }
  ts=new QTextStream(&iofile);
  ts->setCodec("UTF-8");
  parseState=EOH;
  s.clear();
  recordLine=0;
  return NOERROR;
}

bool adif::readRecord()
{
  int pos;
  QString lineIn;
  while(!ts->atEnd())
    {
      switch (parseState)
        {
        case EOH:
          lineIn=ts->readLine();
          s+=lineIn+"\n";
          if((pos=lineIn.indexOf("<EOH>",0,Qt::CaseInsensitive))>=0) //<EOH> in line
            parseState=EOR;
            s.clear();
        break;
        case EOR:
          lineIn=ts->readLine();
          if((pos=lineIn.indexOf("<EOR>",0,Qt::CaseInsensitive))>=0) //<EOR> in line
            {
              recordLine++;
              if(pos>0)
                {
                  s+=lineIn.left(pos);
                  lineIn=lineIn.remove(0,pos+5);
                }
              else
                {
                  lineIn="";
                }
              parse(s);
              s=lineIn;
              return true;
            }
          else
            {
              s+=lineIn;
            }
        }
    }
  return false;
}

bool adif::parse(QString &record)
{
  int i;
  for(i=0;i<NUMBEROFQSOFIELDS;i++) qsoFields[i].clear();
  while(!record.isEmpty()) // while there are more fields to process
    {
      if(!split(record)) //try to get a single adif field
        {
          record="";
          return false;
        }
      if(tag=="") continue; // just skip
      else
        {
          if(!setValue(tag))
            {
              errorString+=QString("unsupported tag %1=%2\n").arg(tag).arg(value);
            }
        }
    }
  return true;
}

bool  adif::setValue(QString tag)
{
  int i;
  for(i=0;i<NUMBEROFQSOFIELDS;i++)
    {
      if(qsoFields[i].tag==tag)
        {
          *qsoFields[i].value=value;
          return true;
        }
    }
  return false;
}

bool adif::split(QString &fieldsLeft)
{
  QString num,stmp;
  int len;
  bool ok;
  int pos1,pos2;
  pos1=fieldsLeft.indexOf("<");
  if(pos1<0)
    {
      errorString+="illegal record construct: in record ending at line ";
      errorString+=QString::number(recordLine)+" ";
      errorString+=fieldsLeft;
      errorString+="\n";
      return false;
    }
  pos2=fieldsLeft.indexOf(">");
  tag=fieldsLeft.mid(pos1+1,pos2-1-pos1);
  fieldsLeft.remove(pos1,pos2+1-pos1);
  pos1=tag.indexOf(":");
  pos2=tag.indexOf(":",pos1+1);
  if(pos2>pos1)
    {
      len=tag.mid(pos1+1,pos2-1-pos1).toUInt(&ok);
    }
  else
    {
      len=tag.mid(pos1+1,tag.length()-pos1+1).toUInt(&ok);
    }
  if(!ok) return false;
  tag.remove(pos1,tag.length()-pos1);
  value=fieldsLeft.left(len);
  fieldsLeft.remove(0,len);

  while (fieldsLeft.left(1)==" " || fieldsLeft.left(1)=="\n")
    {
      fieldsLeft.remove(0,1);
    }
  return true;
}


QString adif::getValue(QString tag)
{
  int i;
  for(i=0;i<NUMBEROFQSOFIELDS;i++)
    {
      if(qsoFields[i].tag==tag)
        {
          return *qsoFields[i].value;
        }
    }
  return QString();
}

