#include "directoriesconfig.h"
#include "ui_directoriesconfig.h"

#include <QFontDialog>


QString importPath;
QString exportPath;
QString templatesPath;
QString fontFamily;
int fontSize;



directoriesConfig::directoriesConfig(QWidget *parent) :baseConfig(parent),
  ui(new Ui::directoriesConfig)
{
  ui->setupUi(this);
  connect(ui->importPathBrowseButton,SIGNAL(clicked()),SLOT(slotBrowseImportPath()));
  connect(ui->exportPathBrowseButton,SIGNAL(clicked()),SLOT(slotBrowseExportPath()));
  connect(ui->templatesPathBrowseButton,SIGNAL(clicked()),SLOT(slotBrowseTemplatesPath()));
  connect(ui->fontSelectPushutton,SIGNAL(clicked()),SLOT(slotFontSelect()));
}

directoriesConfig::~directoriesConfig()
{
  delete ui;
}

void directoriesConfig::readSettings()
{
  QSettings qSettings;
  qSettings.beginGroup("DIRECTORIES");
  importPath=qSettings.value("importPath",QString(getenv("HOME"))+"/").toString();
  exportPath=qSettings.value("exportPath",QString(getenv("HOME"))+"/").toString();
  templatesPath=qSettings.value("templatesPath",QString(getenv("HOME"))+"/").toString();
  fontFamily=qSettings.value("fontFamily",QApplication::font().family()).toString();
  fontSize=qSettings.value("font",QApplication::font().pointSize()).toInt();
  qSettings.endGroup();
  setParams();
}

void directoriesConfig::writeSettings()
{
  QSettings qSettings;
  getParams();
  qSettings.beginGroup("DIRECTORIES");
  qSettings.setValue("importPath",importPath);
  qSettings.setValue("exportPath",exportPath);
  qSettings.setValue("templatesPath",templatesPath);
  qSettings.setValue("fontFamily",fontFamily);
  qSettings.setValue("font",fontSize);
  qSettings.endGroup();
}

void directoriesConfig::getParams()
{
  QString rxImagePathSaved=importPath;
  QString txImagePathSaved=exportPath;
  QString templatesPathSaved=templatesPath;
  getValue(importPath,ui->importPathLineEdit);
  getValue(exportPath,ui->exportPathLineEdit);
  getValue(templatesPath,ui->templatesPathLineEdit);
  getValue(fontFamily,ui->fontLineEdit);
  getValue(fontSize,ui->fontSizeSpinBox);

}

void directoriesConfig::setParams()
{
  setValue(importPath,ui->importPathLineEdit);
  setValue(exportPath,ui->exportPathLineEdit);
  setValue(templatesPath,ui->templatesPathLineEdit);
  setValue(fontFamily,ui->fontLineEdit);
  setValue(fontSize,ui->fontSizeSpinBox);
  QApplication::setFont(QFont(fontFamily,fontSize));
}

/**
  Browse function for path where the rximages are stored
*/


void directoriesConfig::slotBrowseImportPath()
{
  browseDir(ui->importPathLineEdit,importPath);
}

/**
  Browse function for path where the tximages are stored
*/

void directoriesConfig::slotBrowseExportPath()
{
  browseDir(ui->exportPathLineEdit,exportPath);
}

/**
  Browse function for path where the templates are stored
*/

void directoriesConfig::slotBrowseTemplatesPath()
{
  browseDir(ui->templatesPathLineEdit,templatesPath);
}

void directoriesConfig::slotFontSelect()
{
  bool ok;
  QFont qfont=QFontDialog::getFont(&ok,QFont(fontFamily,fontSize),this);
  if(ok)
    {
      fontFamily=qfont.family();
      fontSize=qfont.pointSize();
      QApplication::setFont(qfont);
    }
  setParams();
}

