#include "headerinfoform.h"
#include "appglobal.h"
#include "ui_headerinfoform.h"
#include "utils/supportfunctions.h"
#include <QSettings>



QString callsign;
QString contest;
QString categoryAssisted;
QString categoryBand;
QString categoryMode;
QString categoryOperator;
QString categoryPower;
QString categoryStation;
QString categoryTime;
QString categoryTransmitter;
QString categoryOverlay;
QString certificate;
QString claimedScore;
QString club;
QString email;
QString location;
QString name;
QString address;
QString city;
QString stateProvince;
QString postalCode;
QString country;
QString operators;
QString offtime;
QString soapbox;
QDateTime fromDateTime;
QDateTime toDateTime;
QString soapBox;
bool version3;
bool offTimeEnabled;


QString booleanList[]=
{
  "YES",
  "NO",
  NULL
};


QString contestList[]=
{
  "AP-SPRINT",
  "ARRL-10",
  "ARRL-160",
  "ARRL-DX-CW",
  "ARRL-DX-SSB",
  "ARRL-SS-CW",
  "ARRL-SS-SSB",
  "ARRL-UHF-AUG",
  "ARRL-VHF-JAN",
  "ARRL-VHF-JUN",
  "ARRL-VHF-SEP",
  "ARRL-RTTY",
  "BARTG-RTTY",
  "CQ-160-CW",
  "CQ-160-SSB",
  "CQ-WPX-CW",
  "CQ-WPX-RTTY",
  "CQ-WPX-SSB",
  "CQ-VHF",
  "CQ-WW-CW",
  "CQ-WW-RTTY",
  "CQ-WW-SSB",
  "DARC-WAEDC-CW",
  "DARC-WAEDC-RTTY",
  "DARC-WAEDC-SSB",
  "FCG-FQP",
  "IARU-HF",
  "JIDX-CW",
  "JIDX-SSB",
  "NAQP-CW",
  "NAQP-SSB",
  "NA-SPRINT-CW",
  "NA-SPRINT-SSB",
  "NCCC-CQP",
  "NEQP",
  "OCEANIA-DX-CW",
  "OCEANIA-DX-SSB",
  "RDXC",
  "RSGB-IOTA",
  "SAC-CW",
  "SAC-SSB",
  "STEW-PERRY",
  "TARA-RTTY",
  NULL
};

QString assistedlist[]=
{
  "ASSISTED",
  "NON-ASSISTED",
  NULL
};

QString bandList[]=
{
  "ALL",
  "160M",
  "80M",
  "40M",
  "20M",
  "15M",
  "10M",
  "6M",
  "2M",
  "222",
  "432",
  "902",
  "1.2G",
  "2.3G",
  "3.4G",
  "5.7G",
  "10G",
  "24G",
  "47G",
  "75G",
  "119G",
  "142G",
  "241G",
  "Light",
  "VHF-3-BAND"
  "VHF-FM-ONLY",
  NULL
};

QString modeList[]=
{
  "SSB",
  "CW",
  "RTTY",
  "MIXED",
  NULL
};

QString operatorList[]=
{
  "SINGLE-OP",
  "MULTI-OP",
  "CHECKLOG",
  NULL
};

QString powerList[]=
{
  "HIGH",
  "LOW",
  "QRP",
  NULL
};

QString stationList[]=
{
  "FIXED",
  "MOBILE",
  "PORTABLE",
  "ROVER",
  "ROVER-LIMITED",
  "ROVER-UNLIMITED",
  "EXPEDITION",
  "HQ",
  "SCHOOL",
  NULL
};

QString timeList[]=
{
  "6-HOURS",
  "12-HOURS",
  "24-HOURS",
  NULL
};


QString transmitterList[]=
{
  "ONE",
  "TWO",
  "LIMITED",
  "UNLIMITED",
  "SWL",
  NULL
};

QString overlayList[]=
{
  "CLASSIC",
  "ROOKIE",
  "TB-WIRES",
  "NOVICE-TECH",
  "OVER-50",
  NULL
};


stag tagList[]=
{
  {"START-OF-LOG",          NULL },
  {"END-OF-LOG",            NULL },
  {"CALLSIGN",              NULL },
  {"CONTEST",               contestList},
  {"CATEGORY-ASSISTED",     assistedlist },
  {"CATEGORY-BAND",         bandList },
  {"CATEGORY-MODE",         modeList },
  {"CATEGORY-OPERATOR",     operatorList },
  {"CATEGORY-POWER",        powerList },
  {"CATEGORY-STATION",      stationList },
  {"CATEGORY-TIME",         timeList },
  {"CATEGORY-TRANSMITTER",  transmitterList },
  {"CATEGORY-OVERLAY",      overlayList },
  {"CERTIFICATE",           booleanList },
  {"CLAIMED-SCORE",         NULL },
  {"CLUB",                  NULL },
  {"CREATED-BY",            NULL},
  {"EMAIL",                 NULL },
  {"LOCATION",              NULL },
  {"NAME",                  NULL },
  {"ADDRESS",               NULL },
  {"ADDRESS-CITY",          NULL },
  {"ADDRESS-STATE-PROVINCE", NULL },
  {"ADDRESS-POSTALCODE",     NULL },
  {"ADDRESS-COUNTRY",        NULL },
  {"OPERATORS",              NULL  },
  {"OFFTIME",                NULL },
  {"SOAPBOX",                NULL },
  {"QSO",                    NULL },
  {"X-<anything>",           NULL },
  {"X-QSO",                  NULL },
  {"DEBUG",                  NULL },
  { NULL, NULL }
};



headerInfoForm::headerInfoForm(QWidget *parent) :
  QWidget(parent),
  ui(new Ui::headerInfoForm)
{
  ui->setupUi(this);
  setupComboBoxes();
  readSettings();

}

headerInfoForm::~headerInfoForm()
{
  writeSettings();
  delete ui;
}

void headerInfoForm::setupComboBoxes()
{
  int i;
  for(i=0;tagList[i].tag!=NULL;i++)
    {
      if(tagList[i].tag=="CONTEST")  addComboBoxItem(ui->contestComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-ASSISTED") addComboBoxItem(ui->assistedComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-BAND")     addComboBoxItem(ui->bandComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-MODE")      addComboBoxItem(ui->modeComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-OPERATOR") addComboBoxItem(ui->operatorComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-POWER")    addComboBoxItem(ui->powerComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-STATION")  addComboBoxItem(ui->stationComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-TIME")     addComboBoxItem(ui->timeComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-TRANSMITTER") addComboBoxItem(ui->transmitterComboBox,tagList[i].options);
      if(tagList[i].tag=="CATEGORY-OVERLAY")  addComboBoxItem(ui->overlayComboBox,tagList[i].options);
      if(tagList[i].tag=="CERTIFICATE")       addComboBoxItem(ui->certicateComboBox,tagList[i].options);
    }
}

void headerInfoForm::addComboBoxItem(QComboBox *cmbPtr,QString listPtr[])
{
  int i=0;
  while(listPtr[i]!=NULL)
    {
      cmbPtr->addItem(listPtr[i++]);
    }
}

QStringList headerInfoForm::outputHeaders()
{
  int i;
  getParams();
  headerText.clear();
  if(version3) headerText.append("START-OF-LOG: 3.0");
  else headerText.append("START-OF-LOG: 2.0");
  addLineEditText("CALLSIGN",ui->callsignLineEdit);
  addComboText("CONTEST",ui->contestComboBox);
  if(version3)
    {
      addComboText("CATEGORY-ASSISTED",ui->assistedComboBox);
      addComboText("CATEGORY-BAND",ui->bandComboBox);
      addComboText("CATEGORY-MODE", ui->modeComboBox);
      addComboText("CATEGORY-OPERATOR",ui->operatorComboBox);
      addComboText("CATEGORY-POWER",ui->powerComboBox);
      addComboText("CATEGORY-STATION",ui->stationComboBox);
      addComboText("CATEGORY-TIME",ui->timeComboBox);
      addComboText("CATEGORY-TRANSMITTER",ui->transmitterComboBox);

    }
  else
    {
      addString("CATEGORY",ui->operatorComboBox->currentText()+" "+ui->bandComboBox->currentText()+" "+ui->powerComboBox->currentText());
    }
  addComboText("CATEGORY-OVERLAY", ui->overlayComboBox);
  if(version3) addComboText("CERTIFICATE",ui->certicateComboBox);

  addLineEditText("CLAIMED-SCORE",ui->claimedScoreLineEdit);
  addLineEditText("CLUB",ui->clubLineEdit );
  headerText.append(QString("CREATED-BY: %1").arg(a2cVersion));
  addLineEditText("EMAIL", ui->emailLineEdit );
  addLineEditText("LOCATION", ui->locationLineEdit );
  addLineEditText("NAME",ui->nameLineEdit );


  addLineEditText("ADDRESS", ui->addressLineEdit );
  if(version3)
    {
      addLineEditText("ADDRESS-CITY", ui->cityLineEdit );
      addLineEditText("ADDRESS-STATE-PROVINCE",ui->stateProvinceLineEdit );
      addLineEditText("ADDRESS-POSTALCODE", ui->postalCodeLineEdit );
      addLineEditText("ADDRESS-COUNTRY",ui->countryLineEdit );
    }
  else
    {
      addLineEditText("ADDRESS", ui->cityLineEdit );
      addLineEditText("ADDRESS",ui->stateProvinceLineEdit );
      addLineEditText("ADDRESS", ui->postalCodeLineEdit );
      addLineEditText("ADDRESS",ui->countryLineEdit );
    }
  addLineEditText("OPERATORS", ui->operatorsLineEdit );
  if(version3 && offTimeEnabled)
    {
      //OFFTIME: 2002-03-22 0300 2002-03-22 0743
      headerText.append("OFFTIME: "+fromDateTime.date().toString("yyyy-MM-dd ") +fromDateTime.time().toString("hhmm ")
                        +toDateTime.date().toString("yyyy-MM-dd ") +toDateTime.time().toString("hhmm"));
    }
  QStringList sb;
  sb=ui->soapBoxPlainTextEdit->toPlainText().split("\n");
  for(i=0;i<sb.count();i++)
    {
      headerText.append("SOAPBOX: "+sb.at(i));
    }
    return headerText;
}


void headerInfoForm::addLineEditText(QString label,QLineEdit *lePtr)
{
  if(!lePtr->text().isEmpty()) headerText.append(label+": "+lePtr->text());
}

void headerInfoForm::addComboText(QString label,QComboBox *cbPtr)
{
  if(!cbPtr->currentText().isEmpty()) headerText.append(label+": "+cbPtr->currentText());
}

void headerInfoForm::addString(QString label,QString content)
{
  if(!content.isEmpty()) headerText.append(label+": "+ content);
}

void headerInfoForm::readSettings()
{
  QSettings qSettings;
  qSettings.beginGroup("HEADER");
  version3=qSettings.value("VERSION3","").toBool();
  callsign=qSettings.value("CALLSIGN","").toString();
  contest=qSettings.value("CONTEST","").toString();
  categoryAssisted=qSettings.value("CATEGORY-ASSISTED","").toString();
  categoryBand=qSettings.value("CATEGORY-BAND","").toString();
  categoryMode=qSettings.value("CATEGORY-MODE", "").toString();
  categoryOperator=qSettings.value("CATEGORY-OPERATOR","").toString();
  categoryPower=qSettings.value("CATEGORY-POWER","").toString();
  categoryStation=qSettings.value("CATEGORY-STATION","").toString();
  categoryTime=qSettings.value("CATEGORY-TIME","").toString();
  categoryTransmitter=qSettings.value("CATEGORY-TRANSMITTER","").toString();
  categoryOverlay=qSettings.value("CATEGORY-OVERLAY", "").toString();
  certificate=qSettings.value("CERTIFICATE","").toString();
  claimedScore=qSettings.value("CLAIMED-SCORE","").toString();
  club=qSettings.value("CLUB","").toString();
  email=qSettings.value("EMAIL","").toString();
  location=qSettings.value("LOCATION", "" ).toString();
  name =qSettings.value("NAME","").toString();
  address=qSettings.value("ADDRESS", "").toString();
  city=qSettings.value("ADDRESS-CITY", "").toString();
  stateProvince=qSettings.value("ADDRESS-STATE-PROVINCE","").toString();
  postalCode=qSettings.value("ADDRESS-POSTALCODE","").toString();
  country=qSettings.value("ADDRESS-COUNTRY","").toString();
  operators=qSettings.value("OPERATORS", "").toString();
  soapBox=qSettings.value("SOAPBOX", "").toString();
  fromDateTime=qSettings.value("fromDateTime", QDateTime(QDate(1900,01,01),QTime(0,0,0),Qt::UTC)).toDateTime();
  toDateTime=qSettings.value("toDateTime", QDateTime(QDate(1900,01,01),QTime(0,0,0),Qt::UTC)).toDateTime();
  offTimeEnabled=qSettings.value("OFFTIMEENABLED","").toBool();
  qSettings.endGroup();
  setParams();
}

void headerInfoForm::writeSettings()
{
  QSettings qSettings;
  getParams();
  qSettings.beginGroup("HEADER");
  qSettings.setValue("VERSION3",version3);
  qSettings.setValue("CALLSIGN",callsign);
  qSettings.setValue("CONTEST",contest);
  qSettings.setValue("CATEGORY-ASSISTED",categoryAssisted);
  qSettings.setValue("CATEGORY-BAND",categoryBand);
  qSettings.setValue("CATEGORY-MODE", categoryMode);
  qSettings.setValue("CATEGORY-OPERATOR",categoryOperator);
  qSettings.setValue("CATEGORY-POWER",categoryPower);
  qSettings.setValue("CATEGORY-STATION",categoryStation);
  qSettings.setValue("CATEGORY-TIME",categoryTime);
  qSettings.setValue("CATEGORY-TRANSMITTER",categoryTransmitter);
  qSettings.setValue("CATEGORY-OVERLAY", categoryOverlay);
  qSettings.setValue("CERTIFICATE",certificate);
  qSettings.setValue("CLAIMED-SCORE",claimedScore);
  qSettings.setValue("CLUB",club);
  qSettings.setValue("EMAIL", email);
  qSettings.setValue("LOCATION", location );
  qSettings.setValue("NAME",name );
  qSettings.setValue("ADDRESS", address);
  qSettings.setValue("ADDRESS-CITY", city);
  qSettings.setValue("ADDRESS-STATE-PROVINCE",stateProvince);
  qSettings.setValue("ADDRESS-POSTALCODE",postalCode);
  qSettings.setValue("ADDRESS-COUNTRY",country);
  qSettings.setValue("OPERATORS", operators);
  qSettings.setValue("fromDateTime",fromDateTime);
  qSettings.setValue("toDateTime",toDateTime);
  qSettings.setValue("SOAPBOX", soapBox);
  qSettings.setValue("OFFTIMEENABLED",offTimeEnabled);
  qSettings.endGroup();
}

void headerInfoForm::getParams()
{
  getValue(version3,ui->c3RradioButton);
  getValue(callsign,ui->callsignLineEdit);
  getValue(contest,ui->contestComboBox);
  getValue(categoryAssisted,ui->assistedComboBox);
  getValue(categoryBand,ui->bandComboBox);
  getValue(categoryMode, ui->modeComboBox);
  getValue(categoryOperator,ui->operatorComboBox);
  getValue(categoryPower,ui->powerComboBox);
  getValue(categoryStation,ui->stationComboBox);
  getValue(categoryTime,ui->timeComboBox);
  getValue(categoryTransmitter,ui->transmitterComboBox);
  getValue(categoryOverlay, ui->overlayComboBox);
  getValue(certificate,ui->certicateComboBox);
  getValue(claimedScore,ui->claimedScoreLineEdit);
  getValue(club,ui->clubLineEdit );
  getValue(email, ui->emailLineEdit );
  getValue(location, ui->locationLineEdit );
  getValue( name,ui->nameLineEdit );
  getValue(address, ui->addressLineEdit );
  getValue(city, ui->cityLineEdit );
  getValue(stateProvince,ui->stateProvinceLineEdit );
  getValue(postalCode, ui->postalCodeLineEdit );
  getValue(country,ui->countryLineEdit );
  getValue(operators, ui->operatorsLineEdit );
  fromDateTime=ui->fromDateTimeEdit->dateTime();
  toDateTime=ui->toDateTimeEdit->dateTime();
  getValue(offTimeEnabled,ui->offTimeCheckBox);
}

void headerInfoForm::setParams()
{
  setValue(version3,ui->c3RradioButton);
  setValue(callsign,ui->callsignLineEdit);
  setValue(contest,ui->contestComboBox);
  setValue(categoryAssisted,ui->assistedComboBox);
  setValue(categoryBand,ui->bandComboBox);
  setValue(categoryMode, ui->modeComboBox);
  setValue(categoryOperator,ui->operatorComboBox);
  setValue(categoryPower,ui->powerComboBox);
  setValue(categoryStation,ui->stationComboBox);
  setValue(categoryTime,ui->timeComboBox);
  setValue(categoryTransmitter,ui->transmitterComboBox);
  setValue(categoryOverlay, ui->overlayComboBox);
  setValue(certificate,ui->certicateComboBox);
  setValue(claimedScore,ui->claimedScoreLineEdit);
  setValue(club,ui->clubLineEdit );
  setValue(email, ui->emailLineEdit );
  setValue(location, ui->locationLineEdit );
  setValue( name,ui->nameLineEdit );
  setValue(address, ui->addressLineEdit );
  setValue(city, ui->cityLineEdit );
  setValue(stateProvince,ui->stateProvinceLineEdit );
  setValue(postalCode, ui->postalCodeLineEdit );
  setValue(country,ui->countryLineEdit );
  setValue(operators, ui->operatorsLineEdit );
  ui->fromDateTimeEdit->setDateTime(fromDateTime);
  ui->toDateTimeEdit->setDateTime(toDateTime);
  ui->soapBoxPlainTextEdit->clear();
  ui->soapBoxPlainTextEdit->appendPlainText(soapBox);
  setValue(offTimeEnabled,ui->offTimeCheckBox);
}


