#include "mainwindow.h"
#include "appglobal.h"
#include "configdialog.h"
#include "adif.h"
#include <QDate>

#include "ui_mainwindow.h"


#include <QMessageBox>

MainWindow::MainWindow(QWidget *parent) :
  QMainWindow(parent),
  ui(new Ui::MainWindow)
{
  ui->setupUi(this);
  connect(ui->actionExit,SIGNAL(triggered()),this, SLOT(slotExit()));
  connect(ui->actionConfigure,SIGNAL(triggered()),this, SLOT(slotConfigure()));
  connect(ui->processPushButton,SIGNAL(clicked()),SLOT(slotProcess()));
  connect(ui->savePushButton,SIGNAL(clicked()),SLOT(slotSave()));
  connect(ui->actionAboutA2C, SIGNAL(triggered()),SLOT(slotAboutA2C()));
  connect(ui->actionAboutQt, SIGNAL(triggered()),SLOT(slotAboutQt()));


  configDialogPtr=new configDialog(this);
  readSettings();
  ui->tabWidget->setCurrentIndex(0);

}

MainWindow::~MainWindow()
{
  delete ui;
}

void MainWindow::readSettings()
{
  //  QScreen *screen=QGuiApplication::screens().at(0);
  //  int maxWidth=screen->availableGeometry().width();
  //  int maxHeight=screen->availableGeometry().height();

  QSettings qSettings;
  qSettings.beginGroup("MAIN");
  int windowWidth = qSettings.value("windowWidth", 460 ).toInt();
  int windowHeight = qSettings.value("windowHeight", 530 ).toInt();
  int windowX = qSettings.value( "windowX", -1 ).toInt();
  int windowY = qSettings.value( "windowY", -1 ).toInt();
  resize( windowWidth, windowHeight );
  if ( windowX != -1 || windowY != -1 )
    {
      move( windowX, windowY );
    }
  qSettings.endGroup();
  configDialogPtr->readSettings();
  ui->templateWidget->readSettings();
}

void MainWindow::writeSettings()
{
  QSettings qSettings;
  qSettings.beginGroup("MAIN");
  qSettings.setValue( "windowWidth", width() );
  qSettings.setValue( "windowHeight", height() );
  qSettings.setValue( "windowX", x() );
  qSettings.setValue( "windowY", y() );
  qSettings.endGroup();
  ui->headerWidget->writeSettings();
  ui->templateWidget->writeSettings();
  configDialogPtr->writeSettings();

}

void MainWindow::slotProcess()
{
  ui->templateWidget->process(ui->headerWidget,ui->outputWidget);
}

void MainWindow::slotSave()
{
  ui->templateWidget->save(ui->outputWidget);
}

void MainWindow::slotExit()
{
  int exit;
  exit=QMessageBox::information(this, tr("Quit..."),tr("Do your really want to quit ADIF2CABRILLO ?"), QMessageBox::Ok, QMessageBox::Cancel);
  if(exit==QMessageBox::Ok)
    {
      writeSettings();
      QApplication::quit();
    }

}


void MainWindow::slotAboutA2C()
{
  QString temp=tr("ADIF2CABRILLO \nVersion: ") + MAJORVERSION + MINORVERSION ;
  temp += QString("\n http://users.telenet.be/on4qz \n(c) 2014-%1 -- Johan Maes - ON4QZ\n%2").arg(QDate::currentDate().toString("yyyy")).arg(shortName);



  QMessageBox::about(this,tr("About..."),temp);

}

void MainWindow::slotAboutQt()
{
  QMessageBox::aboutQt(this,tr("About..."));

}

void MainWindow::slotConfigure()
{

  if(configDialogPtr->exec()==QDialog::Accepted)
    {
    }
}
