#include "templateform.h"
#include "configparams.h"
#include "dirdialog.h"
#include "supportfunctions.h"

#include <QHBoxLayout>
#include <QSpacerItem>
#include <QLabel>
#include <QSpacerItem>
#include <QPushButton>
#include <QDir>
#include <QDebug>
#include <QMessageBox>

static QList<int> sortList;

static bool lessThan(const QStringList &v1,const QStringList & v2)
{
  int i;
  for(i=0;i<sortList.count();i++)
    {
      if(v1.at(sortList.at(i))<v2.at(sortList.at(i))) return true;
      if(v1.at(sortList.at(i))>v2.at(sortList.at(i))) return false;
    }
  return false;
}


templateForm::templateForm(QWidget *parent) : QWidget(parent)
{
  setup();
}

templateForm::~templateForm()
{

}

void templateForm::setup()
{

  QVBoxLayout *verticalLayout;
  QHBoxLayout *horizontalLayout;
  QHBoxLayout *horizontalLayout2;
  QHBoxLayout *horizontalLayout3;
  QLabel*templateLabel;
  QLabel*sortLabel;
  QLabel*notesLabel;
  QSpacerItem *verticalSpacer;
  QPushButton *savePushButton;


  verticalLayout = new QVBoxLayout(this);
  horizontalLayout = new QHBoxLayout();
  horizontalLayout2 = new QHBoxLayout();
  horizontalLayout3 = new QHBoxLayout();
  templateLabel=new QLabel();
  templateLabel->setText("Template");
  templateComboBox=new QComboBox(this);
  savePushButton=new QPushButton("Save",this);
  horizontalLayout=new QHBoxLayout();
  horizontalLayout->addWidget(templateLabel);
  horizontalLayout->addWidget(templateComboBox);
  horizontalLayout->addWidget(savePushButton);
  horizontalLayout->setStretch(1, 1);
  sortLabel=new QLabel();
  sortLabel->setText("Sort");
  sortLineEdit=new QLineEdit(this);
  horizontalLayout2->addWidget(sortLabel);
  horizontalLayout2->addWidget(sortLineEdit);
  notesLabel=new QLabel();
  notesLabel->setText("Notes");
  notesPlainText=new QPlainTextEdit(this);
  horizontalLayout3->addWidget(notesLabel);
  horizontalLayout3->addWidget(notesPlainText);
  gridLayout = new QGridLayout();
  gridLayout->setVerticalSpacing(1);
  setupFields();
  verticalSpacer = new QSpacerItem(20, 322, QSizePolicy::Minimum, QSizePolicy::Expanding);
  verticalLayout->addLayout(horizontalLayout);
  verticalLayout->addLayout(horizontalLayout3);
  verticalLayout->addLayout(gridLayout);
  verticalLayout->addLayout(horizontalLayout2);
  verticalLayout->addItem(verticalSpacer);
  verticalLayout->setSpacing(1);
  connect(savePushButton,SIGNAL(clicked()),SLOT(saveTemplate()));
  connect(templateComboBox,SIGNAL(currentIndexChanged(int)),SLOT(slotTemplateChange(int)));

}

void templateForm::setupFields()
{
  int i;
  QLabel *lblPtr;
  lblPtr=new QLabel(this);
  lblPtr->setText("FIELD#");
  lblPtr->setAlignment(Qt::AlignCenter);
  gridLayout->addWidget(lblPtr, 0, 0, 1, 1);

  lblPtr=new QLabel(this);
  lblPtr->setText("INPUT");
  lblPtr->setAlignment(Qt::AlignCenter);
  gridLayout->addWidget(lblPtr, 0, 1, 1, 1);
  lblPtr=new QLabel(this);
  lblPtr->setText("LENGTH");
  lblPtr->setAlignment(Qt::AlignCenter);
  gridLayout->addWidget(lblPtr, 0, 2, 1, 1);
  lblPtr=new QLabel(this);
  lblPtr->setText("PARSE");
  lblPtr->setAlignment(Qt::AlignCenter);
  gridLayout->addWidget(lblPtr, 0, 3, 1, 1);
  for(i=0;i<MAXFIELDS;i++)
    {
      lblPtr=new QLabel(this);
      lblPtr->setText(QString::number(i+1));
      lblPtr->setAlignment(Qt::AlignCenter);
      gridLayout->addWidget(lblPtr, i+1, 0, 1, 1);
      templateFieldArray[i].cbPtr=new QComboBox(this);
      templateFieldArray[i].cbPtr->setEditable(true);
      gridLayout->addWidget(templateFieldArray[i].cbPtr, i+1, 1, 1, 1);
      templateFieldArray[i].lengthPtr=new QSpinBox(this);
      gridLayout->addWidget(templateFieldArray[i].lengthPtr, i+1, 2, 1, 1);
      templateFieldArray[i].parsePtr=new QLineEdit(this);
      gridLayout->addWidget(templateFieldArray[i].parsePtr, i+1, 3, 1, 1);
    }
  for(i=0;i<MAXFIELDS;i++)
    {
      templateFieldArray[i].cbPtr->addItem("");
      templateFieldArray[i].cbPtr->addItems(adf.getFieldList(true));
    }
}

void templateForm::process(headerInfoForm *headerPtr, QPlainTextEdit *textWindowPtr)
{
  int i;
  QString tmp;
  QStringList sl;

  textWindowPtr->clear();
  qsoList.clear();
  sl.clear();
  if(adf.openFile()!=NOERROR)
    {
      return;
    }

  sl=headerPtr->outputHeaders();
  for(i=0;i<sl.count();i++)
    {
      textWindowPtr->appendPlainText(sl.at(i));
    }

  while(adf.readRecord())
    {
      qsoStrList.clear();
      for(i=0;i<MAXFIELDS;i++)
        {
          appendFields(i);
        }
      qsoList.append(qsoStrList);
    }
  sorting();
  for(i=0;i<qsoList.count();i++)
    {
      textWindowPtr->appendPlainText("QSO: "+qsoList.at(i).join(" "));
    }
  textWindowPtr->appendPlainText("END-OF-LOG:");
}

void templateForm::sorting()
{
  int i;
  QStringList sl;
  sortList.clear();
  sl=sortLineEdit->text().split(",");
  for(i=0;i<sl.count();i++)
    {
      int k;
      k=sl.at(i).toInt()-1;
      if(k>=0 && k<=MAXFIELDS && !templateFieldArray[k].cbPtr->currentText().isEmpty()) sortList << k;
    }
  std::sort(qsoList.begin(), qsoList.end(),lessThan);
}

void templateForm::appendFields(int idx)
{
  QString qsoStr;
  QString inputStr;
  int length;
  QString parseStr;

  inputStr=templateFieldArray[idx].cbPtr->currentText();
  length=templateFieldArray[idx].lengthPtr->value();
  parseStr=templateFieldArray[idx].parsePtr->text();

  if(!inputStr.isEmpty())
    {
      // do we have plain text or an ADIF field;
      if(inputStr.left(1)=="<")
        {
          inputStr.chop(1);
          inputStr.remove(0,1);
//          qsoStr=" "+parse(parseStr,adf.getValue(inputStr),length).leftJustified(length);
           qsoStr=parse(parseStr,adf.getValue(inputStr),length).leftJustified(length);

        }
      else
        {
          qsoStr=inputStr.leftJustified(length).toUpper();
        }
    }
  qsoStrList.append(qsoStr);
}

QString  templateForm::parse(QString pStr,QString src,int length)
{
  int i;
  QStringList parseStrList=pStr.split(";");
  QString rStr=src;
  for(i=0;i<parseStrList.count();i++)
    {

      if(parseStrList.at(i)=="#D8")
        {
          rStr=src.left(4)+"-"+src.mid(4,2)+"-"+src.right(2);
        }
      else if(parseStrList.at(i)=="#M2K")
        {
          float khz;
          khz=src.toFloat() *1000;
          rStr=QString::number((int)khz).rightJustified(length);
        }
      else if(parseStrList.at(i)=="#LZ")
        {
          while(rStr.length()<length) rStr.prepend("0");
        }
      else if(parseStrList.at(i).left(1)>="0" &&  parseStrList.at(i).left(1)<="9")
        {
          QStringList sl=parseStrList.at(i).split(",");
          if(sl.count()==2)
            {
              rStr=src.mid(sl.at(0).toInt()-1,sl.at(1).toInt());
            }
        }
    }
  return rStr.toUpper();
}



void templateForm::save(QPlainTextEdit *textWindowPtr)
{
  QString t;
  QFile iofile;
  enum QFile::FileError er;
  dirDialog dd(mainWindowPtr,"Export Cabrillo");
  iofn=dd.saveFileName(templatesPath,"Cabrillo(*.cbr *.CBR);;ALL(*.*)","CBR");
  if(iofn.isEmpty()) return;
  iofile.setFileName(iofn);
  if(!iofile.open( QIODevice::WriteOnly ))
    {
      QString ferrorStr;
      er=iofile.error();
      fileError(er,ferrorStr);
      QMessageBox::critical(0,"Save Products",QString("Could not open %1 for writing\n").arg(iofn)+ferrorStr);
      return ;
    }
  QTextStream ts(&iofile);
  ts.setCodec("UTF-8");
  t=textWindowPtr->toPlainText();
  t.replace("\n","\r\n");
  ts<<t;
  iofile.close();
  return;

}

eerrorType templateForm::loadTemplate(QString iofn,QString name)
{
  int i;
  if (( iofn.isEmpty() ) || (iofn==NULL) ) return CANCELED;
  QSettings qSettings(iofn, QSettings::NativeFormat);
  qSettings.beginGroup("TEMPLATEENTRIES");
  for(i=0;i<MAXFIELDS;i++)
    {
#if QT_VERSION >= 0x050000
      templateFieldArray[i].cbPtr->setCurrentText(getEntry(&qSettings,"input",i));
#else
      templateFieldArray[i].cbPtr->setItemText(templateFieldArray[i].cbPtr->currentIndex(), getEntry(&qSettings,"input",i));
#endif
      templateFieldArray[i].lengthPtr->setValue(getEntry(&qSettings,"length",i).toInt());
      templateFieldArray[i].parsePtr->setText(getEntry(&qSettings,"parse",i));
    }
  sortLineEdit->setText(qSettings.value("SORT","").toString());
  notesPlainText->clear();
  notesPlainText->appendPlainText(qSettings.value("NOTES","").toString());
  qSettings.endGroup();
  templateFile=name;
  return NOERROR;
}

QString templateForm::getEntry(QSettings *qs,QString param,int idx)
{
  QString paramidx;
  paramidx=param+QString::number(idx);
  return qs->value(paramidx,"").toString();
}

void  templateForm::setEntry(QSettings *qs,QString param,int idx,QString value)
{
  QString paramidx;
  paramidx=param+QString::number(idx);
  qs->setValue(paramidx,value);
}


eerrorType templateForm::saveTemplate()
{
  int i;
  dirDialog dd(mainWindowPtr,"Export Template");
  iofn=dd.saveFileName(templatesPath,"Cabrillo Template(*.templ);;ALL(*.*)","templ");
  //  dirDialog dd(mainWindowPtr,"Export Cabrillo");
  //  iofn=dd.openFileName(templatesPath,"Cabrillo(*.cbr *.CBR);;ALL(*.*)");

  if (( iofn.isEmpty() ) || (iofn==NULL) ) return CANCELED;
  QSettings qSettings(iofn, QSettings::NativeFormat);
  qSettings.beginGroup("TEMPLATEENTRIES");
  for(i=0;i<MAXFIELDS;i++)
    {
      setEntry(&qSettings,"input",i,templateFieldArray[i].cbPtr->currentText());
      setEntry(&qSettings,"length",i,QString::number(templateFieldArray[i].lengthPtr->value()));
      setEntry(&qSettings,"parse",i,templateFieldArray[i].parsePtr->text());
    }
  qSettings.setValue("SORT",sortLineEdit->text());
  qSettings.setValue("NOTES",notesPlainText->toPlainText());
  qSettings.endGroup();
  return NOERROR;
}


void templateForm::readSettings()
{
  int i;
  QDir dir(templatesPath);
  // get files from template directory
  templateFiles=dir.entryInfoList(QStringList("*.templ"),QDir::Files | QDir::NoSymLinks );
  for (i=0;i<templateFiles.count();i++)
    {
      templateComboBox->addItem(templateFiles.at(i).baseName());
    }
  QSettings qSettings;
  qSettings.beginGroup("TEMPLATE");
  templateFile=qSettings.value("templateFile","").toString();
  qSettings.endGroup();
  if(!templateFile.isEmpty())
    {
#if QT_VERSION >= 0x050000
      templateComboBox->setCurrentText(templateFile);
#else
      templateComboBox->setItemText(templateComboBox->currentIndex(), templateFile);
#endif
      loadTemplate(templateFiles.at(templateComboBox->currentIndex()).absoluteFilePath(),templateFiles.at(templateComboBox->currentIndex()).baseName() );
    }
  else
    {
      if(templateFiles.count()>0)
        {
          loadTemplate(templateFiles.at(0).absoluteFilePath(),templateFiles.at(0).baseName());
        }
    }
}

void templateForm::writeSettings()
{
  QSettings qSettings;
  qSettings.beginGroup("TEMPLATE");
  qSettings.setValue("templateFile",templateFile);
  qSettings.endGroup();
}

void templateForm::slotTemplateChange(int idx)
{
  qDebug() << "template change" << idx;
  if(idx<templateFiles.count())
    {
      loadTemplate(templateFiles.at(idx).absoluteFilePath(),templateFiles.at(idx).baseName());
    }
}

